#!/usr/bin/env php
<?php
/* Copyright (C) 2021      Open-Dsi <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       scripts/process_box.php
 *  \ingroup    banking4dolibarr
 *  \brief      Execute process by CLI
 */
if (!defined('NOTOKENRENEWAL')) define('NOTOKENRENEWAL', '1'); // Disables token renewal
if (!defined('NOREQUIREMENU')) define('NOREQUIREMENU', '1');
if (!defined('NOREQUIREHTML')) define('NOREQUIREHTML', '1');
if (!defined('NOREQUIREAJAX')) define('NOREQUIREAJAX', '1');
if (!defined('NOLOGIN')) define('NOLOGIN', '1');

$sapi_type = php_sapi_name();
$script_file = basename(__FILE__);
$path = dirname(__FILE__) . '/';

// Test if batch mode
if (substr($sapi_type, 0, 3) == 'cgi') {
	print "Error: You are using PHP for CGI. To execute " . $script_file . " from command line, you must use PHP for CLI mode.\n";
	exit(-1);
}

// Check parameters
if (!isset($argv[4]) || !$argv[4]) {
	print "Usage: $script_file id topic entity json_params\n";
	exit(-1);
}

// Change this following line to use the correct relative path (../, ../../, etc)
$res = 0;
if (!$res && file_exists("../../master.inc.php")) $res = @include '../../master.inc.php';            // to work if your module directory is into a subdir of root htdocs directory
if (!$res && file_exists("../../../master.inc.php")) $res = @include '../../../master.inc.php';        // to work if your module directory is into a subdir of root htdocs directory
if (!$res) {
	print "Include of master fails\n";
	exit(-1);
}
require_once DOL_DOCUMENT_ROOT . '/core/lib/functions.lib.php';
require_once DOL_DOCUMENT_ROOT . '/compta/bank/class/account.class.php';
dol_include_once('/banking4dolibarr/class/budgetinsight.class.php', 'BudgetInsight');
dol_include_once('/banking4dolibarr/class/banking4dolibarrprocessbox.class.php', 'Banking4DolibarrProcessBox');

@set_time_limit(0);

// Global variables
$version = DOL_VERSION;
$now = dol_now();
$error = 0;
$id = $argv[1] > 0 ? $argv[1] : 0;
$topic = trim($argv[2]);
$entity = trim($argv[3]);
$parameters = json_decode(base64_decode($argv[4]), true);


/*
 * Main
 */

dol_syslog($script_file . ' - Running ...', LOG_DEBUG);

// Check parameters
if (!in_array($topic, array('auto_reconciliation', 'refresh_bank_records'))) {
	print "Error: Bad topic\n";
	$error++;
}
if (empty($id)) {
	print "Error: Bad id\n";
	$error++;
}

if (!$error) {
	$langs->load('banking4dolibarr@banking4dolibarr');
	$process_box = new Banking4DolibarrProcessBox($db);

	$process_info = $process_box->getProcessInfo($id, $topic);
	if ($process_info['status']) {
		// don't launch multiple script for a same site and topic
		exit(0);
	}

	$result = $process_box->setProcessInfo($id, $topic, getmypid(), $now);
	if ($result < 0) {
		$process_box->addEventMessage($id, $topic, "Error: {$process_box->errorsToString()}", true, 'error');
		$error++;
	}

	if (!$error && !is_array($parameters)) {
		$process_box->addEventMessage($id, $topic, "Error: Fail decode parameters", true, 'error');
		$error++;
	}

	// Load config
	if (!$error) {
		$result = $process_box->setEntityValues($entity);
		if ($result < 0) {
			$process_box->addEventMessage($id, $topic, "Error: load conf (Entity: " . $entity . ")", true, 'error');
			$error++;
		}
	}

	// Fetch user
	if (!$error) {
		$user_id = $parameters['user_id'] > 0 ? $parameters['user_id'] : 0;
		if (empty($user_id)) {
			$process_box->addEventMessage($id, $topic, "Error: User ID not provided", true, 'error');
			$error++;

		}
	}
	if (!$error) {
		require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
		$user = new User($db);
		$result = $user->fetch($user_id, '', '', 1);
		if ($result == 0) {
			$process_box->addEventMessage($id, $topic, "Error: User (ID: $user_id) not found", true, 'error');
			$error++;
		} elseif ($result < 0) {
			$process_box->addEventMessage($id, $topic, "Error: Fail fetch user (ID: $user_id) : {$user->errorsToString()}", true, 'error');
			$error++;
		} else {
			$user->getrights();
		}
	}

	// Reload langs if need
	if (!$error) {
		$langcode = (GETPOST('lang', 'aZ09', 1) ? GETPOST('lang', 'aZ09', 1) : (empty($conf->global->MAIN_LANG_DEFAULT) ? 'auto' : $conf->global->MAIN_LANG_DEFAULT));
		if (defined('MAIN_LANG_DEFAULT')) {
			$langcode = constant('MAIN_LANG_DEFAULT');
		}
		$newlang = '';
		if (!empty($user->conf->MAIN_LANG_DEFAULT) && $user->conf->MAIN_LANG_DEFAULT != $langs->defaultlang) {
			$newlang = $user->conf->MAIN_LANG_DEFAULT;
		} elseif (!empty($langcode) && $langcode != $langs->defaultlang) {
			$newlang = $langcode;
		}
		if (!empty($newlang)) {
			$langs = new Translate("", $conf);
			$langs->setDefaultLang($newlang);
			$langs->loadLangs(array('main', 'banks', 'banking4dolibarr@banking4dolibarr'));
		}
	}

	// Reload mysoc
	if (!$error && isset($mysoc)) {
		$mysoc->setMysoc($conf);
		// We set some specific default values according to country
		if ($mysoc->country_code == 'DE' && !isset($conf->global->MAIN_INVERT_SENDER_RECIPIENT)) {
			// For DE, we need to invert our address with customer address
			$conf->global->MAIN_INVERT_SENDER_RECIPIENT = 1;
		}
		if ($mysoc->country_code == 'FR' && !isset($conf->global->INVOICE_CATEGORY_OF_OPERATION)) {
			// For FR, default value of option to show category of operations is on by default. Decret n°2099-1299 2022-10-07
			$conf->global->INVOICE_CATEGORY_OF_OPERATION = 1;
		}
		if ($mysoc->country_code == 'FR' && !isset($conf->global->INVOICE_DISABLE_REPLACEMENT)) {
			// For FR, the replacement invoice type is not allowed.
			// From an accounting point of view, this creates holes in the numbering of the invoice.
			// This is very problematic during a fiscal control.
			$conf->global->INVOICE_DISABLE_REPLACEMENT = 1;
		}
		if ($mysoc->country_code == 'GR' && !isset($conf->global->INVOICE_DISABLE_REPLACEMENT)) {
			// The replacement invoice type is not allowed in Greece.
			$conf->global->INVOICE_DISABLE_REPLACEMENT = 1;
		}
		if ($mysoc->country_code == 'GR' && !isset($conf->global->INVOICE_DISABLE_DEPOSIT)) {
			// The deposit invoice type is not allowed in Greece.
			$conf->global->INVOICE_DISABLE_DEPOSIT = 1;
		}
		if (($mysoc->localtax1_assuj || $mysoc->localtax2_assuj) && !isset($conf->global->MAIN_NO_INPUT_PRICE_WITH_TAX)) {
			// For countries using the 2nd or 3rd tax, we disable input/edit of lines using the price including tax (because 2nb and 3rd tax not yet taken into account).
			// Work In Progress to support all taxes into unit price entry when MAIN_UNIT_PRICE_WITH_TAX_IS_FOR_ALL_TAXES is set.
			$conf->global->MAIN_NO_INPUT_PRICE_WITH_TAX = 1;
		}
	}

	if (!$error) {
		$langs->load('banks');
		$account = new Account($db);
		$result = $account->fetch($id);
		if ($result == 0) {
			$process_box->addEventMessage($id, $topic, "Error: Bank account (ID: $id) not found", true, 'error');
			$error++;
		} elseif ($result < 0) {
			$process_box->addEventMessage($id, $topic, "Error: Fail fetch bank account (ID: $id) : {$account->errorsToString()}", true, 'error');
			$error++;
		}
	}

	if (!$error && !checkUserAccessToObject($user, array('banque'), $account->id, 'bank_account&bank_account', '', '', 'rowid')) {
		$process_box->addEventMessage($id, $topic, $langs->trans('ErrorForbidden'), true, 'error');
		$error++;
	}

	if (!$error && $account->clos != 0) {
		$process_box->addEventMessage($id, $topic, $langs->trans('Banking4DolibarrErrorBankAccountNotOpen'), true, 'error');
		$error++;
	}

	if (!$error && !empty($parameters['process_box_initial_text'])) {
		$process_box->addEventMessage($id, $topic, trim($parameters['process_box_initial_text']), true);
	}

	if (!$error && $topic == 'auto_reconciliation') {
		if (empty($user->rights->banque->consolidate)) {
			$process_box->addEventMessage($id, $topic, $langs->trans('ErrorForbidden'), true, 'error');
			$error++;
		}

		if (!$error) {
			$budgetinsight = new BudgetInsight($db);
			$statement_number = !empty($parameters['statement_number']) ? $parameters['statement_number'] : '';

			$result = $budgetinsight->autoLinkBankRecords($user, $id, $statement_number);
			if ($result < 0) {
				$process_box->addEventMessage($id, $topic, '<span style="color: red;">' . $budgetinsight->errorsToString(), true, 'error');
				$error++;
			}
		}
	}

	// Remove process info
	sleep(2); // For be sure the process is detected as started so don't delete it
	$result = $process_box->setProcessInfo($id, $topic, 0);
	if ($result < 0) {
		$process_box->addEventMessage($id, $topic, "Error: {$process_box->errorsToString()}", true, 'error');
		$error++;
	}
}

dol_syslog($script_file . ' - Stopped ...', LOG_DEBUG);

$db->close();
exit($error ? -1 : 0);
